@@ -308,6 +308,20 @@ macro_roman_bold_small_italic_handler (YelpManParser *parser, gchar *macro, GSLi
 	else
 	    parser->ins = parser_append_node (parser, b);
 	
+	if (ptr->next) {
+	    gchar *tmp = ptr->next->data;
+	    
+	    if (tmp[0] == '(' && g_ascii_isdigit (tmp[1]) &&
+		(tmp[2] == ')' || (g_ascii_isalpha (tmp[2]) && tmp[3] == ')'))) {
+		tmp = g_strconcat (ptr->data, " ", tmp, NULL);
+		parser_append_given_text_handle_escapes (parser, tmp, TRUE);
+		g_free (tmp);
+		parser->ins = parser->ins->parent;
+		ptr = ptr->next->next;
+		continue;
+	    }  
+	}
+
 	parser_append_given_text_handle_escapes (parser, ptr->data, TRUE);
 	parser->ins = parser->ins->parent;
 	
@@ -706,11 +720,10 @@ macro_reference_handler (YelpManParser *parser, gchar *macro, GSList *args)
 	if (args && args->data) {
 	    basename = g_strrstr((const gchar *)args->data, "/");
 
-	    basename++;
-	    
-	    if (basename)
+	    if (basename) {
+		basename++;
 		link = g_strdup_printf ("man:%s", basename);
-	    else {
+	    } else {
 		link = g_strdup_printf ("man:%s", (const gchar *)args->data);
 		basename = (gchar *)args->data;
 	    }
@@ -1454,6 +1467,9 @@ parser_append_given_text_handle_escapes (YelpManParser *parser, gchar *text, gbo
 	        if (*(ptr) == 'R') {
 		    parser_append_given_text (parser, "®");
 		    ptr++;
+		} else if (*(ptr) == '=') {
+		    parser_append_given_text (parser, "--");
+		    ptr++;
 	        } else if (*(ptr) == '(') {
 		    ptr++;
 		    if ((ptr - baseptr) > len || *ptr == '\0') break;
@@ -1523,25 +1539,33 @@ parser_append_given_text_handle_escapes (YelpManParser *parser, gchar *text, gbo
 	}
         else if ((make_links) && (*ptr == '(')) {
 	    gchar *space_pos;
-	    gchar *tmp_cur;
 	    gchar *url;
 	    gchar  c;
+	    gchar *name_end;
+            gchar *num_start;
+            gchar *num_end;
+
 	   
 	    space_pos = ptr;
 	    
 	    while (space_pos != anc && *(space_pos - 1) != ' ') {
 		space_pos--;
 	    }
+	    name_end = space_pos;
 	    
 	    if (space_pos != ptr &&
 	        g_ascii_isdigit(*(ptr+1)) &&
-		*(ptr+2) == ')') {
+		(*(ptr+2) == ')' || (g_ascii_isalpha (*(ptr+2)) && *(ptr+3) == ')'))) {
+                num_start = ptr;
+		if (*(ptr+2) == ')')
+		    num_end = ptr + 2;
+                else
+		    num_end = ptr + 3;
 	    
 		ptr+=3;
 	    
 		parser_ensure_P (parser);
 	    
-		tmp_cur = ptr;
 		ptr = space_pos;
 	    
 		c = (*ptr);
@@ -1549,12 +1573,13 @@ parser_append_given_text_handle_escapes (YelpManParser *parser, gchar *text, gbo
 		parser_append_given_text (parser, anc);
 		*ptr = c;
 		anc = ptr;
-	    
-		ptr = tmp_cur;
-	    
-		c = *(ptr);
-		*(ptr) = '\0';
-		url = g_strdup_printf ("man:%s", anc);
+		ptr = num_start;
+
+		c = *name_end;
+		*name_end = '\0';
+                *num_end = '\0';
+		url = g_strdup_printf ("man:%s(%s)", anc, num_start + 1);
+
 	    
 		parser->ins = parser_append_node (parser, "UR");
 	
@@ -1565,7 +1590,8 @@ parser_append_given_text_handle_escapes (YelpManParser *parser, gchar *text, gbo
 		parser_append_given_text (parser, anc);
 		parser->ins = parser->ins->parent;
 	    
-		*(ptr) = c;
+		*name_end = c;
+                *num_end = ')';
 		anc = ptr;
 	    
 		g_free (url);
