@@ -1008,17 +1008,19 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 	doc = priv->current_document;
     } else {
 	g_free (priv->base_uri);
+	priv->base_uri = NULL;
+
 	switch (type) {
 	case YELP_RRN_TYPE_TOC:
 	    doc = yelp_toc_get ();
 	    priv->base_uri = g_strdup ("file:///fakefile");
 	    break;
 	case YELP_RRN_TYPE_MAN:
-	    priv->base_uri = g_strdup_printf ("file:/%s", real_uri);
+	    priv->base_uri = g_filename_to_uri (real_uri, NULL, NULL);
 	    doc = yelp_man_new (real_uri);
 	    break;
 	case YELP_RRN_TYPE_INFO:
-	    priv->base_uri = g_strdup_printf ("file:/%s", real_uri);
+	    priv->base_uri = g_filename_to_uri (real_uri, NULL, NULL);
 	    if (!frag_id) {
 		frag_id = g_strdup ("Top");
 	    } else {
@@ -1027,7 +1029,7 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 	    doc = yelp_info_new (real_uri);
 	    break;
 	case YELP_RRN_TYPE_DOC:
-	    priv->base_uri = g_strdup_printf ("file:/%s", real_uri);
+	    priv->base_uri = g_filename_to_uri (real_uri, NULL, NULL);
 	    doc = yelp_docbook_new (real_uri);
 	    break;
 	case YELP_RRN_TYPE_SEARCH:
@@ -1037,6 +1039,7 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 	    break;
 	case YELP_RRN_TYPE_HTML:
 	case YELP_RRN_TYPE_XHTML:
+	    priv->base_uri = g_strdup ("file:///fakefile");
 	    window_do_load_html (window, real_uri, frag_id, type, TRUE);
 	    break;
 	case YELP_RRN_TYPE_EXTERNAL:
@@ -1046,6 +1049,9 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 		gchar *cmd = NULL;
 		gint status = 0;
 		GError *error = NULL;
+
+		priv->base_uri = g_strdup ("file:///fakefile");
+
 		cmd = g_strdup_printf ("gnome-open %s", uri);
 		if (!g_spawn_command_line_sync (cmd, &str_stdout, &str_stderr, &status, &error)) {
 		    g_free (error);
@@ -1063,8 +1069,10 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 		    return;
 		}
 	    }
+	    break;
 
 	default:
+	    priv->base_uri = g_strdup ("file:///fakefile");
 	    break;
 	}
     }
@@ -1540,6 +1548,7 @@ window_do_load_html (YelpWindow    *window,
     gchar            buffer[BUFFER_SIZE];
     GtkAction       *action;
     gchar *real_uri = NULL;
+    gchar *base_uri = NULL;
 
     gboolean  handled = TRUE;
 
@@ -1575,10 +1584,13 @@ window_do_load_html (YelpWindow    *window,
 	goto done;
     }
 
-    if (frag_id)
-	real_uri = g_strdup_printf ("file:/%s#%s", uri, frag_id);
-    else
-	real_uri = g_strdup_printf ("file:/%s", uri);
+    base_uri = g_filename_to_uri (real_uri, NULL, NULL);
+    if (frag_id) {
+	real_uri = g_strconcat (base_uri, "#", frag_id);
+	g_free (base_uri);
+    } else {
+	real_uri = base_uri;
+    }
     yelp_html_set_base_uri (priv->html_view, real_uri);
 
     switch (type) {
@@ -2646,12 +2658,20 @@ tree_model_iter_following (GtkTreeModel  *model,
 static gboolean
 window_write_html (YelpLoadData *data)
 {
+    gchar *uri = NULL;
     gsize read;
     YelpHtml *html = data->window->priv->html_view;
     gchar contents[BUFFER_SIZE];
     
     /* Use a silly fake URI to stop gecko doing silly things */
-    yelp_html_set_base_uri (html, data->window->priv->base_uri);
+    if (data->window->priv->current_frag)
+	uri = g_strdup_printf ("%s#%s", data->window->priv->base_uri, 
+			       data->window->priv->current_frag);
+    else
+	uri = g_strdup (data->window->priv->base_uri);
+    
+    yelp_html_set_base_uri (html, uri);
+    g_free (uri);
     yelp_html_open_stream (html, "application/xhtml+xml");
     
     do {
